# R code adapted by Richard G. Clegg 2005
# Code originally by Vern Paxson
# from http://ita.ee.lbl.gov/html/contrib/fft-fgn.html
# Fast Approximation of Self-Similar Network Traffic, V. Paxson, 
# technical report LBL-36750/UC-405, April 1995.

# Usage fbmgen(n,H) generates a sample path of n points of fractional
# Brownian motion with Hurst H.
# fgn (n,H) generates a sample path of n points of fractional 
# Gaussian noise 

trafficgen <-function (n, mean, H, pktlen=1, rate=0, filename="", len= 0)

# generate approximately n packets with mean being the ratio of packets and
# inter packet gaps, H is the Hurst parameter and rate is the bits/sec to be
# generated

# returns a series of 1s and 0s
# n is the number of ones to be generated (approximately)
# mean is the approximate fractions of 1s to be generated
# H is the Hurst parameter for the fBm
# If len is given then len digits are generated
# If a filename, pktlen and rate are given then the 1s and 0s are
# written to a file as packets and interpacket gaps assuming a fixed packet size
# and the given rate in bits/unit time
# The file will contain the delta time and the packet length

{
    if (len > 0) {
      fgn<-fgngen(len,H)
    } else {
      len<-2*floor(n/(mean*2))
      fgn<-fgngen(len,H)
    }
    z<-cumsum(pmax(0,fgn))
    ystep<-max(z)/length(z)
    ystep<-ystep/mean
    if (rate > 0) {
      tstep <-pktlen/rate
    } else {
      tstep <- 1
    }
    fptr<-0
    if (filename != "") {
       fptr<-file(filename,"w")
    }
    tot<-0
    dt<- 0
    j<-1
    y<-array(dim=length(z))
    for (i in (1:length(z))) {
#      cat(tot, z[i],ystep,"\n")
      if(tot - z[i] < 0) {
        y[i]<-1
        tot<- tot+ystep  
        if (fptr!= 0) {
          cat (dt,pktlen,"\n",file=fptr)
        }
        dt<-0
      } else {
      
        y[i]<-0
      }
      dt<- dt + tstep    
    }
    if (fptr != 0) {
       close(fptr)
    }
    y
} 

fbmgen <- function (n, H)
# uses fgn-gen to generate n points of fractional
# brownian motion
{
   
    z<-fgngen(n,H)
    
    x<-cumsum(z)
    x
}

fgngen <-
function(n, H)
{
# Returns a Fourier-generated sample path
# of a "self similar" process, consisting
# of n points and Hurst parameter H
# (n should be even).
  if ( (n%%2) != 0) {
    stop ("n should be even\n")
    
  }
  if (H < 0.5 || H >= 1) {
    stop("H should be in (0.5,1)\n")
  
  }

  n <- n/2
  lambda <- ((1:n)*pi)/n

  # Approximate ideal power spectrum.
  f <- FGN.spectrum(lambda, H)

  # Adjust for estimating power
  # spectrum via periodogram.
  f <- f * rexp(n)

  # Construct corresponding complex
  # numbers with random phase.
  z <- complex(modulus = sqrt(f),
               argument = 2*pi*runif(n))

  # Last element should have zero phase.
  z[n] <- abs(z[n])

  # Expand z to correspond to a Fourier
  # transform of a real-valued signal.
  zprime <- c(0, z, Conj(rev(z)[-1]))

  # Inverse FFT gives sample path.
  Re(fft(zprime, inv=T))
}

FGN.spectrum <- function(lambda, H)
{
# Returns an approximation of the power
# spectrum for fractional Gaussian noise
# at the given frequencies lambda and
# the given Hurst parameter H.

  2 * sin(pi*H) * gamma(2*H+1) *
  (1-cos(lambda)) *
  (lambda^(-2*H-1) +
   FGN.B.est(lambda, H))
}

FGN.B.est <- function(lambda, H)
{
# Returns the estimate for
# B(lambda,H).

  d <- -2*H - 1
  dprime <-  -2*H
  a <- function(lambda,k) 2*k*pi+lambda
  b <- function(lambda,k) 2*k*pi-lambda
  a1 <- a(lambda,1)
  b1 <- b(lambda,1)
  a2 <- a(lambda,2)
  b2 <- b(lambda,2)
  a3 <- a(lambda,3)
  b3 <- b(lambda,3)
  a4 <- a(lambda,4)
  b4 <- b(lambda,4)
  a1^d+b1^d+a2^d+b2^d+a3^d+b3^d +
  (a3^dprime+b3^dprime +
   a4^dprime+b4^ dprime)/(8*pi*H)
}
